@@ -1,16 +1,21 @@
 import asyncio
 import os
 import shutil
+import time
 import typing
-from enum import Enum, IntEnum
+from datetime import datetime, timezone
+from enum import Enum
 from pathlib import Path, PurePath
 from typing import Dict, List, TypedDict
 
 from loguru import logger
 
+from primitive.agent.pxe import pxe_boot
+from primitive.utils.asgiref.sync import sync_to_async
 from primitive.utils.cache import get_artifacts_cache, get_logs_cache, get_sources_cache
 from primitive.utils.logging import fmt, log_context
 from primitive.utils.psutil import kill_process_and_children
+from primitive.utils.ssh import wait_for_ssh
 
 if typing.TYPE_CHECKING:
     import primitive.client
@@ -31,12 +36,6 @@ class JobConfig(TypedDict):
     stores: List[str]
 
 
-# NOTE This must match FailureLevel subclass in JobSettings model
-class FailureLevel(IntEnum):
-    ERROR = 1
-    WARNING = 2
-
-
 class LogLevel(Enum):
     INFO = "INFO"
     ERROR = "ERROR"
@@ -48,12 +47,16 @@ class Runner:
         self,
         primitive: "primitive.client.Primitive",
         job_run: Dict,
+        target_hardware_id: str | None = None,
         max_log_size: int = 10 * 1024 * 1024,  # 10MB
     ) -> None:
         self.primitive = primitive
         self.job = job_run["job"]
         self.job_run = job_run
         self.job_settings = job_run["jobSettings"]
+        self.target_hardware_id = target_hardware_id
+        self.target_hardware: dict | None = None
+        self.target_hardware_secret = None
         self.config = job_run["jobSettings"]["config"]
         self.initial_env = {}
         self.modified_env = {}
@@ -70,6 +73,17 @@ class Runner:
                 backtrace=True,
             )
 
+        if self.target_hardware_id is not None:
+            self.target_hardware = self.primitive.hardware.get_hardware_details(
+                id=self.target_hardware_id
+            )
+            target_hardware_secret = self.primitive.hardware.get_hardware_secret(
+                hardware_id=self.target_hardware_id
+            )
+            self.target_hardware_secret = {
+                k: v for k, v in target_hardware_secret.items() if v is not None
+            }
+
     @log_context(label="setup")
     def setup(self) -> None:
         # Attempt to download the job source code
@@ -100,6 +114,18 @@ class Runner:
             **self.initial_env,
             **self.primitive.jobs.get_job_secrets_for_job_run(self.job_run["id"]),
         }
+
+        # Set Primitive-related environment variables that the current client has
+        if not os.environ.get("PRIMITIVE_HOST") and self.primitive.host is not None:
+            self.initial_env["PRIMITIVE_HOST"] = str(self.primitive.host)
+        if not os.environ.get("PRIMITIVE_TOKEN") and self.primitive.token is not None:
+            self.initial_env["PRIMITIVE_TOKEN"] = str(self.primitive.token)
+        if (
+            not os.environ.get("PRIMITIVE_TRANSPORT")
+            and self.primitive.transport is not None
+        ):
+            self.initial_env["PRIMITIVE_TRANSPORT"] = str(self.primitive.transport)
+
         self.initial_env["PRIMITIVE_SOURCE_DIR"] = str(self.source_dir)
         self.initial_env["PRIMITIVE_GIT_SHA"] = str(self.job_run["gitCommit"]["sha"])
         self.initial_env["PRIMITIVE_GIT_BRANCH"] = str(
@@ -223,18 +249,140 @@ class Runner:
     async def run_task(self, task: Task) -> bool:
         logger.info(f"Running step '{task['label']}'")
         commands = task["cmd"].strip().split("\n")
+
         for i, cmd in enumerate(commands):
-            # Adding an additional echo and utilizing stdbuf to force line buffering
-            # This ensures that the environment variables and starting delimiter are
-            # always in a new chunk, vastly simplifying our parsing logic
-            args = ["/bin/bash", "-c", cmd]
+            if cmd.strip() == "":
+                continue
+            if cmd.strip().startswith("#"):
+                logger.debug(f"Skipping comment line: {cmd.strip()}")
+                continue
+            if cmd == "oobpowercycle":
+                logger.info("Performing out-of-band power cycle")
+                from primitive.network.redfish import RedfishClient
+
+                bmc_host = self.target_hardware_secret.get("bmcHostname", None)
+                bmc_username = self.target_hardware_secret.get("bmcUsername", None)
+                bmc_password = self.target_hardware_secret.get("bmcPassword", "")
+
+                if bmc_host is None:
+                    logger.error(
+                        "No BMC host found in target hardware secret for out-of-band power cycle"
+                    )
+                    return True
+                if bmc_username is None:
+                    logger.error(
+                        "No BMC username found in target hardware secret for out-of-band power cycle"
+                    )
+                    return True
+
+                redfish = RedfishClient(
+                    host=bmc_host, username=bmc_username, password=bmc_password
+                )
+                redfish.compute_system_reset(system_id="1", reset_type="ForceRestart")
+                if self.target_hardware_id:
+                    await self.primitive.hardware.aupdate_hardware(
+                        hardware_id=self.target_hardware_id,
+                        is_online=False,
+                        is_rebooting=True,
+                        start_rebooting_at=str(datetime.now(timezone.utc)),
+                    )
+                    logger.info(
+                        "Box rebooting, waiting 30 seconds before beginning SSH connection."
+                    )
+                    time.sleep(30)
+                    wait_for_ssh(
+                        hostname=self.target_hardware_secret.get("hostname"),
+                        username=self.target_hardware_secret.get("username"),
+                        password=self.target_hardware_secret.get("password"),
+                        port=22,
+                    )
+                    logger.info("Reboot successful, SSH is now available")
+                    await self.primitive.hardware.aupdate_hardware(
+                        hardware_id=self.target_hardware_id,
+                        is_online=True,
+                        is_rebooting=False,
+                    )
+                continue
+
+            if (
+                cmd == "pxeboot"
+                and self.target_hardware is not None
+                and self.target_hardware_secret
+            ):
+                logger.info("Setting next boot to PXE and rebooting")
+
+                await sync_to_async(
+                    self.primitive.operating_systems._prepare_operating_system_for_pxe
+                )(
+                    organization_id=self.target_hardware.get("organization", {}).get(
+                        "id"
+                    ),
+                    operating_system_slug=self.target_hardware.get(
+                        "defaultOperatingSystem", {}
+                    ).get("slug"),
+                    iso_filename=self.target_hardware.get("defaultOperatingSystem")
+                    .get("isoFile")
+                    .get("fileName"),
+                    controller_hostname="192.168.10.1",  # TODO dynamically obtain this later
+                    target_ip_address=self.target_hardware_secret.get("hostname", ""),
+                )
+                pxe_boot(target_hardware_secret=self.target_hardware_secret)
+
+                if self.target_hardware_id:
+                    await self.primitive.hardware.aupdate_hardware(
+                        hardware_id=self.target_hardware_id,
+                        is_online=False,
+                        is_rebooting=True,
+                        start_rebooting_at=str(datetime.now(timezone.utc)),
+                    )
+                    logger.info(
+                        "Box rebooting, waiting 30 seconds before beginning SSH connection."
+                    )
+                    time.sleep(30)
+                    wait_for_ssh(
+                        hostname=self.target_hardware_secret.get("hostname"),
+                        username=self.target_hardware_secret.get("username"),
+                        password=self.target_hardware_secret.get("password"),
+                        port=22,
+                    )
+                    logger.info("PXE boot successful, SSH is now available")
+                    await self.primitive.hardware.aupdate_hardware(
+                        hardware_id=self.target_hardware_id,
+                        is_online=True,
+                        is_rebooting=False,
+                    )
+                continue
+
+            # if target hardware id and target hardware secret were populated, ssh onto the target to run the command
+            if self.target_hardware_secret:
+                username = self.target_hardware_secret.get("username")
+                password = self.target_hardware_secret.get("password")
+                hostname = self.target_hardware_secret.get("hostname")
+                command_args = [
+                    "sshpass",
+                    "-p",
+                    password,
+                    "ssh",
+                    "-o",
+                    "StrictHostKeyChecking=no",
+                    "-o",
+                    "UserKnownHostsFile=/dev/null",
+                    "-o",
+                    "IdentitiesOnly=yes",
+                    f"{username}@{hostname}",
+                    "--",
+                    f"{cmd}",
+                ]
+                print(" ".join(command_args))
+            else:
+                command_args = ["/bin/bash", "--login", "-c", cmd]
 
             logger.info(
                 f"Executing command {i + 1}/{len(commands)}: {cmd} at {self.source_dir / task.get('workdir', '')}"
             )
 
             process = await asyncio.create_subprocess_exec(
-                *args,
+                *command_args,
                 env=self.modified_env,
                 cwd=str(Path(self.source_dir / task.get("workdir", ""))),
                 stdout=asyncio.subprocess.PIPE,
@@ -298,15 +446,16 @@ class Runner:
 
     @log_context(label="cleanup")
     def cleanup(self) -> None:
-        for glob in self.config.get("stores", []):
-            # Glob relative to the source directory
-            matches = self.source_dir.rglob(glob)
+        if stores := self.config.get("stores"):
+            for glob in stores:
+                # Glob relative to the source directory
+                matches = self.source_dir.rglob(glob)
 
-            for match in matches:
-                relative_path = PurePath(match).relative_to(self.source_dir)
-                dest = Path(get_artifacts_cache(self.job_run["id"]) / relative_path)
-                dest.parent.mkdir(parents=True, exist_ok=True)
-                Path(match).replace(dest)
+                for match in matches:
+                    relative_path = PurePath(match).relative_to(self.source_dir)
+                    dest = Path(get_artifacts_cache(self.job_run["id"]) / relative_path)
+                    dest.parent.mkdir(parents=True, exist_ok=True)
+                    Path(match).replace(dest)
 
         shutil.rmtree(path=self.source_dir)
 
