@@ -1,23 +1,16 @@
 from gql import gql
 
-from ..utils.config import read_config_file, update_config_file
-from ..utils.auth import guard
-
 from primitive.utils.actions import BaseAction
 
+from ..utils.auth import guard
+from ..utils.config import read_config_file, update_config_file
+from .graphql.queries import whoami_query
+
 
 class Auth(BaseAction):
     @guard
     def whoami(self):
-        query = gql(
-            """
-            query whoami {
-                whoami {
-                    username
-                }
-            }
-        """
-        )
+        query = gql(whoami_query)
 
         result = self.primitive.session.execute(query, get_execution_result=True)
 
@@ -25,14 +18,12 @@ class Auth(BaseAction):
 
     def setup_config(
         self,
-        username: str,
         token: str,
         host: str = "api.primitive.tech",
         transport: str = "https",
     ):
         full_config = read_config_file()
         new_host_config = {
-            "username": username,
             "token": token,
             "transport": transport,
         }
