@@ -17,6 +17,8 @@ from .organizations.commands import cli as organizations_commands
 from .projects.commands import cli as projects_commands
 from .reservations.commands import cli as reservations_commands
 from .monitor.commands import cli as monitor_commands
+from .network.commands import cli as network_commands
+from .operating_systems.commands import cli as operating_system_commands
 
 
 @click.group()
@@ -73,6 +75,8 @@ cli.add_command(projects_commands, "projects")
 cli.add_command(reservations_commands, "reservations")
 cli.add_command(exec_commands, "exec")
 cli.add_command(monitor_commands, "monitor")
+cli.add_command(network_commands, "network")
+cli.add_command(operating_system_commands, "operating-systems")
 
 if __name__ == "__main__":
     cli(obj={})
