@@ -1,3 +1,4 @@
+import os
 from typing import Optional
 
 from gql import Client
@@ -5,6 +6,8 @@ from loguru import logger
 from rich.logging import RichHandler
 from rich.traceback import install
 
+from primitive.messaging.provider import MessagingProvider
+
 from .agent.actions import Agent
 from .auth.actions import Auth
 from .daemons.actions import Daemons
@@ -14,6 +17,8 @@ from .git.actions import Git
 from .hardware.actions import Hardware
 from .jobs.actions import Jobs
 from .monitor.actions import Monitor
+from .network.actions import Network
+from .operating_systems.actions import OperatingSystems
 from .organizations.actions import Organizations
 from .projects.actions import Projects
 from .provisioning.actions import Provisioning
@@ -24,13 +29,32 @@ from .utils.config import read_config_file
 class Primitive:
     def __init__(
         self,
-        host: str = "api.primitive.tech",
         DEBUG: bool = False,
         JSON: bool = False,
+        host: Optional[str] = None,
         token: Optional[str] = None,
         transport: Optional[str] = None,
     ) -> None:
-        self.host: str = host
+        # if no host supplied by user, check the environment variables first
+        if host is None:
+            # if user didn't supply host and no PRIMITIVE_HOST environment variable
+            # use the default production API host
+            self.host = os.getenv("PRIMITIVE_HOST", "api.primitive.tech")
+        else:
+            self.host = host
+
+        # if no host supplied by user, check the environment variables first
+        if token is None:
+            self.token = os.getenv("PRIMITIVE_TOKEN", None)
+        else:
+            self.token = token
+
+        # if no host supplied by user, check the environment variables first
+        if transport is None:
+            self.transport = os.getenv("PRIMITIVE_TRANSPORT", None)
+        else:
+            self.transport = transport
+
         self.session: Optional[Client] = None
         self.DEBUG: bool = DEBUG
         self.JSON: bool = JSON
@@ -80,6 +104,8 @@ class Primitive:
         else:
             self.host_config = {"username": "", "token": token, "transport": transport}
 
+        self.messaging: MessagingProvider = MessagingProvider(self)
+
         self.auth: Auth = Auth(self)
         self.organizations: Organizations = Organizations(self)
         self.projects: Projects = Projects(self)
@@ -93,6 +119,8 @@ class Primitive:
         self.exec: Exec = Exec(self)
         self.provisioning: Provisioning = Provisioning(self)
         self.monitor: Monitor = Monitor(self)
+        self.network: Network = Network(self)
+        self.operating_systems: OperatingSystems = OperatingSystems(self)
 
     def get_host_config(self):
         self.full_config = read_config_file()
