@@ -31,10 +31,6 @@ fragment JobRunFragment on JobRun {
   jobSettings {
     containerArgs
     rootDirectory
-    parseLogs
-    parseStderr
-    failureLevel
-    repositoryFilename
     config
   }
   gitCommit {
@@ -42,6 +38,10 @@ fragment JobRunFragment on JobRun {
     branch
     repoFullName
   }
+  executionHardware {
+    id
+    pk
+  }
 }
 """
 
@@ -51,5 +51,6 @@ fragment JobRunStatusFragment on JobRun {
     status
     conclusion
     parentPid
+    jobRunNumber
 }
 """
